import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:flutter/material.dart';
import 'package:jiffy/jiffy.dart';

import 'loadable_state_event.dart';
import 'loadable_state_state.dart';

class LoadableStateBloc extends Bloc<LoadableStateEvent, LoadableStateState> {
  late StreamSubscription<List<ConnectivityResult>> _updateStream;
  void Function()? reFetch;

  LoadableStateBloc() : super(const LoadableStateState(connections: null)) {
    on<ConnectivityChanged>((event, emit) {
      emit(event.state);
      if(connectivityStatusKnown() && isConnected()) {
        if(reFetch == null) return;
        reFetch!();
      }
    });

    emitConnectivity(List<ConnectivityResult> result) => add(ConnectivityChanged(LoadableStateState(connections: result)));

    Connectivity().checkConnectivity().then(emitConnectivity);
    _updateStream = Connectivity().onConnectivityChanged.listen(emitConnectivity);
  }

  bool connectivityStatusKnown() => state.connections != null;
  bool isConnected() => !(state.connections?.contains(ConnectivityResult.none) ?? true);
  bool allowRetry() => reFetch != null;

  IconData connectionIcon() => connectivityStatusKnown()
      ? isConnected()
        ? Icons.nearby_error
        : Icons.signal_wifi_connected_no_internet_4
      : Icons.device_unknown;

  Color connectionColor(BuildContext context) => connectivityStatusKnown() && !isConnected()
      ? Colors.grey.shade600
      : Theme.of(context).primaryColor;

  String connectionText({int? lastUpdated}) => connectivityStatusKnown()
      ? isConnected()
        ? 'Verbindung fehlgeschlagen'
        : 'Offline${lastUpdated == null ? '' : ' - Stand von ${Jiffy.parseFromMillisecondsSinceEpoch(lastUpdated).fromNow()}'}'
      : 'Unbekannte Fehlerursache';

  @override
  Future<void> close() {
    _updateStream.cancel();
    return super.close();
  }
}
