import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import '../../../../../widget/infoDialog.dart';
import '../bloc/loadable_state_bloc.dart';

class LoadableStateErrorBar extends StatelessWidget {
  final bool visible;
  final String? message;
  final int? lastUpdated;
  const LoadableStateErrorBar({required this.visible, this.message, this.lastUpdated, super.key});

  final Duration animationDuration = const Duration(milliseconds: 200);

  @override
  Widget build(BuildContext context) => AnimatedSize(
    duration: animationDuration,
    child: AnimatedSwitcher(
        duration: animationDuration,
        transitionBuilder: (Widget child, Animation<double> animation) => SlideTransition(
          position: Tween<Offset>(
            begin: const Offset(0.0, -1.0),
            end: Offset.zero,
          ).animate(animation),
          child: child,
        ),
        child: Visibility(
            key: Key(visible.hashCode.toString()),
            visible: visible,
            replacement: const SizedBox(width: double.infinity),
            child: Builder(
              builder: (context) {
                var bloc = context.watch<LoadableStateBloc>();
                return InkWell(
                  onTap: () {
                    if(!bloc.isConnected()) return;
                    InfoDialog.show(context, 'Exception: ${message.toString()}');
                  },
                  child: Container(
                    height: 20,
                    decoration: BoxDecoration(
                      color: bloc.connectionColor(context),
                    ),
                    child: LoadableStateErrorBarText(lastUpdated: lastUpdated),
                  ),
                );
              },
            )
        )
    ),
  );
}

class LoadableStateErrorBarText extends StatefulWidget {
  final int? lastUpdated;
  const LoadableStateErrorBarText({required this.lastUpdated, super.key});

  @override
  State<LoadableStateErrorBarText> createState() => _LoadableStateErrorBarTextState();
}

class _LoadableStateErrorBarTextState extends State<LoadableStateErrorBarText> {
  late Timer _rebuildTimer;
  @override
  void initState() {
    _rebuildTimer = Timer.periodic(const Duration(seconds: 10), (timer) => setState(() {}));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    var bloc = context.watch<LoadableStateBloc>();

    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Icon(bloc.connectionIcon(), size: 14),
        const SizedBox(width: 10),
        Text(bloc.connectionText(lastUpdated: widget.lastUpdated), style: const TextStyle(fontSize: 12))
      ],
    );
  }

  @override
  void dispose() {
    _rebuildTimer.cancel();
    super.dispose();
  }
}
