import 'package:flutter/material.dart';
import 'package:jiffy/jiffy.dart';

import '../../../../../widget/animatedTime.dart';
import '../../../../../widget/list_view_util.dart';
import '../../../../../widget/centeredLeading.dart';
import '../../../../../widget/debug/debugTile.dart';
import '../../../../../widget/string_extensions.dart';
import '../../../infrastructure/loadableState/loadable_state.dart';
import '../../../infrastructure/loadableState/view/loadable_state_consumer.dart';
import '../../../infrastructure/utilityWidgets/bloc_module.dart';
import '../bloc/holidays_bloc.dart';
import '../bloc/holidays_event.dart';
import '../bloc/holidays_state.dart';

class HolidaysView extends StatelessWidget {
  const HolidaysView({super.key});

  @override
  Widget build(BuildContext context) => BlocModule<HolidaysBloc, LoadableState<HolidaysState>>(
    create: (context) => HolidaysBloc(),
    autoRebuild: true,
    child: (context, bloc, state) {
      void showDisclaimer() {
        showDialog(context: context, builder: (context) => AlertDialog(
          title: const Text('Richtigkeit und Bereitstellung der Daten'),
          content: const Text(''
              'Sämtliche Datumsangaben sind ohne Gewähr.\n'
              'Ich übernehme weder Verantwortung für die Richtigkeit der Daten noch hafte ich für wirtschaftliche Schäden die aus der Verwendung dieser Daten entstehen können.\n\n'
              'Die Daten stammen von https://ferien-api.de/'),
          actions: [
            TextButton(child: const Text('Okay'), onPressed: () => Navigator.of(context).pop()),
          ],
        ));
      }

      return Scaffold(
        appBar: AppBar(
          title: const Text('Schulferien in Hessen'),
          actions: [
            IconButton(
              icon: const Icon(Icons.info_outline),
              onPressed: showDisclaimer,
            ),
            PopupMenuButton<bool>(
              initialValue: bloc.showPastHolidays(),
              icon: const Icon(Icons.history),
              itemBuilder: (context) => [true, false].map((e) => PopupMenuItem<bool>(
                  value: e,
                  enabled: e != bloc.showPastHolidays(),
                  child: Row(
                    children: [
                      Icon(e ? Icons.history_outlined : Icons.history_toggle_off_outlined, color: Theme.of(context).colorScheme.onSurface),
                      const SizedBox(width: 15),
                      Text(e ? 'Alle anzeigen' : 'Nur zukünftige anzeigen')
                    ],
                  )
              )).toList(),
              onSelected: (e) => bloc.add(SetPastHolidaysVisible(e)),
            ),
          ],
        ),
        body: LoadableStateConsumer<HolidaysBloc, HolidaysState>(
          onLoad: (state) {
            if(state.showDisclaimer) showDisclaimer();
            bloc.add(DisclaimerDismissed());
          },
          child: (state, loading) => ListViewUtil.fromList<Holiday>(bloc.getHolidays(), (holiday) {
            var holidayType = holiday.name.split(' ').first.capitalize();
            String formatDate(String date) => Jiffy.parse(date).format(pattern: 'dd.MM.yyyy');
            String getYear(String date, {String format = 'yyyy'}) => Jiffy.parse(date).format(pattern: format);

            String getHolidayYear(String startDate, String endDate) => getYear(startDate) == getYear(endDate)
                  ? getYear(startDate)
                  : '${getYear(startDate)}/${getYear(endDate, format: 'yy')}';

            return ListTile(
              leading: const CenteredLeading(Icon(Icons.calendar_month)),
              title: Text('$holidayType ${getHolidayYear(holiday.start, holiday.end)}'),
              subtitle: Text('${formatDate(holiday.start)} - ${formatDate(holiday.end)}'),
              onTap: () => showDialog(context: context, builder: (context) => SimpleDialog(
                title: Text('$holidayType ${holiday.year} in Hessen'),
                children: [
                  ListTile(
                    leading: const CenteredLeading(Icon(Icons.signpost_outlined)),
                    title: Text(holiday.name.capitalize()),
                    subtitle: Text(holiday.slug.capitalize()),
                  ),
                  ListTile(
                    leading: const Icon(Icons.date_range_outlined),
                    title: Text('vom ${formatDate(holiday.start)}'),
                  ),
                  ListTile(
                    leading: const Icon(Icons.date_range_outlined),
                    title: Text('bis zum ${formatDate(holiday.end)}'),
                  ),
                  Visibility(
                    visible: !DateTime.parse(holiday.start).difference(DateTime.now()).isNegative,
                    replacement: ListTile(
                      leading: const CenteredLeading(Icon(Icons.content_paste_search_outlined)),
                      title: Text(Jiffy.parse(holiday.start).fromNow()),
                    ),
                    child: ListTile(
                      leading: const CenteredLeading(Icon(Icons.timer_outlined)),
                      title: AnimatedTime(callback: () => DateTime.parse(holiday.start).difference(DateTime.now())),
                      subtitle: Text(Jiffy.parse(holiday.start).fromNow()),
                    ),
                  ),
                  DebugTile(context).jsonData(holiday.toJson()),
                ],
              )),
              trailing: const Icon(Icons.arrow_right),
            );
          }),
        ),
      );
    },
  );
}
